import os
import json
import csv
import pydantic
import requests
from pathlib import Path
import logging
from time import sleep
from datetime import (
    datetime,
    timedelta
)

logging.basicConfig(
    level=os.getenv('LOG_LEVEL', 'INFO'),
    format='%(message)s'
)


class ResponseStatus(pydantic.BaseModel):
    cached: bool
    date_time: datetime
    elapsed_time: timedelta
    url: str

    class Config:
        json_encoders = {
            datetime: lambda v: v.isoformat()
        }


class InputOptions(pydantic.BaseModel):
    url: str
    output_file: Path = Path('/tmp/cdn-report.csv')
    time_step: str = '1m'
    http_verb: str = 'GET'
    timeout: int = 60
    header_name = 'x-cache'


def append_record(response_status: ResponseStatus, record_file: Path):
    record_path = record_file.expanduser()
    record_path.parent.mkdir(parents=True, exist_ok=True)

    record_row = json.loads(response_status.json())

    with record_path.open('a') as csvfile:
        fieldnames = list(sorted(record_row.keys()))
        writer = csv.DictWriter(csvfile, fieldnames=fieldnames)
        writer.writerow(record_row)


def check_endpoint_aws_cloudfront(headers: dict, header_name='x-cache') -> bool:
    headers_lower_case = {key.lower(): value for key, value in headers.items()}
    cache_header_value = headers_lower_case.get(header_name)

    if not cache_header_value:
        raise KeyError(f'{header_name} not found in response headers')

    if cache_header_value.lower() == 'hit from cloudfront':
        status = True
    else:
        status = False

    return status


def request_endpoint(url: str, timeout_seconds=60, http_verb='GET') -> (dict, float):
    response = requests.request(http_verb, url=url, timeout=timeout_seconds, allow_redirects=False)
    headers = response.headers.copy()
    elapsed_time = response.elapsed

    return headers, elapsed_time


def parse_time_step(time_step: str) -> timedelta:
    unit_character = time_step[-1].lower()
    time_numbers = time_step[:-1]

    abbreviation = {
        's': 'seconds',
        'm': 'minutes',
        'd': 'days'
    }

    if not unit_character.isalpha():
        unit_character = 's'

    if not time_numbers.isnumeric():
        raise ValueError(f'value {time_numbers} is not number')

    unit_arg = abbreviation[unit_character]

    return timedelta(**{unit_arg: int(time_numbers)})


def main_routine(input_parameters: InputOptions):
    step_time_delta = parse_time_step(input_parameters.time_step)

    while True:
        headers, elapsed_time = request_endpoint(
            url=input_parameters.url,
            timeout_seconds=input_parameters.timeout,
            http_verb=input_parameters.http_verb
        )
        requests_time = datetime.now()
        request_status = check_endpoint_aws_cloudfront(headers, input_parameters.header_name)

        try:
            response_status = ResponseStatus(
                date_time=requests_time,
                elapsed_time=elapsed_time,
                cached=request_status,
                url=input_parameters.url
            )
        except requests.RequestException as e:
            logging.error(f'requests error {e}')
            sleep(5)
            continue

        append_record(response_status, input_parameters.output_file)
        logging.info(response_status.json())

        sleep(step_time_delta.seconds)


def main(input_parameters: InputOptions):
    try:
        main_routine(input_parameters)
    except KeyboardInterrupt:
        pass



from . import domain
import argparse
import sys


__version__ = '0.1.6'


def get_input_options_from_cli() -> domain.InputOptions:
    """
    example use:
        cdn-test --url=https://aws.amazon.com/pt/cloudfront/ --http-verb=GET --time-step=10s --output-file=
    """
    parser = argparse.ArgumentParser()

    if len(sys.argv) < 2:
        parser.print_help()
        parser.exit(status=1)

    parser.add_argument(
        '--url', '-u', type=str,
        help='URL to verified cache in cloudfront'
    )
    parser.add_argument(
        '--time-step', '-s', default='1m', nargs='?',
        help='time interval between requests'
    )
    parser.add_argument(
        '--http-verb', '-x', default='GET', nargs='?',
        help='HTTP verb utilized for requests to URL'
    )
    parser.add_argument(
        '--header-name', default='x-cache', nargs='?',
        help='response header name that contains "Miss from cloudfront" or "Hit from cloudfront"',
    )
    parser.add_argument(
        '--output-file', '-f', default='/tmp/cdn-test.json', nargs='?',
        help='file path to save records',
    )
    parser.add_argument(
        '--version', '-v', action='version', version=f'{__version__}\n',
        help='show cdn-test version'
    )

    args_namespace = parser.parse_args()
    args = vars(args_namespace)
    input_options = domain.InputOptions(**args)

    return input_options


def entrypoint():
    input_parameters = get_input_options_from_cli()
    domain.main(input_parameters)


if __name__ == '__main__':
    entrypoint()



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['cdn_test']

package_data = \
{'': ['*']}

install_requires = \
['pydantic>=1.8.2,<2.0.0', 'requests>=2.26.0,<3.0.0']

entry_points = \
{'console_scripts': ['cdn-test = cdn_test.cli:entrypoint']}

setup_kwargs = {
    'name': 'cdn-test',
    'version': '0.1.6',
    'description': 'CLI tool to test CDN',
    'long_description': '.. image:: https://badge.fury.io/py/cdn-test.svg\n    :target: https://badge.fury.io/py/cdn-test\n\n.. image:: https://travis-ci.com/augustoliks/cdn-test.svg?branch=main\n    :target: https://travis-ci.com/github/augustoliks/cdn-test\n\ncdn-test\n========\n\nCLI tool created to automate tests for verify time cache in web-site delivered by CDN - CloudFront.\n\nHow to Install\n--------------\n\nThe ``cdn-test`` package is hosted in Python Package Index (PyPI). To\ninstall ``cnd-test``, it\'s recommend create a ``virtualenv`` and install the package using ``pip``. Example:\n\n.. code-block:: bash\n\n    $ virtualenv --python=$(which python3) venv\n    $ source venv/bin/activate\n    $ pip3 install cdn-test\n\nManual\n------\n\n.. code-block:: bash\n\n    $ cdn-test -h\n\n    usage: cli.py [-h] [--url URL] [--time-step [TIME_STEP]] [--http-verb [HTTP_VERB]] [--header-name [HEADER_NAME]] [--output-file [OUTPUT_FILE]] [--version]\n\n    options:\n      -h, --help            show this help message and exit\n      --url URL, -u URL     URL to verified cache in cloudfront\n      --time-step [TIME_STEP], -s [TIME_STEP]\n                            time interval between requests\n      --http-verb [HTTP_VERB], -x [HTTP_VERB]\n                            HTTP verb utilized for requests to URL\n      --header-name [HEADER_NAME]\n                            response header name that contains "Miss from cloudfront" or "Hit from cloudfront"\n      --output-file [OUTPUT_FILE], -f [OUTPUT_FILE]\n                            file path to save records\n      --version, -v         show cdn-test version\n\nExamples\n--------\n\nEach 10 seconds, it will be made a request with ``GET`` HTTP verb to web-site hosted by cloudformation with follow URL ``https://aws.amazon.com/pt/cloudfront/``. The request history will be save on ``~/cdn-report.json``.\n\n.. code-block:: bash\n\n    $ cdn-test --url=https://aws.amazon.com/pt/cloudfront/ --http-verb=GET --time-step=10s --output-file=~/cdn-report.json\n\nPackage Struct\n--------------\n\n.. code-block:: bash\n\n    ├── src\n    │   ├── cdn_test            # source code directory\n    │   ├── __init__.py         # module definition file\n    │   ├── cli.py              # parser cli parameters\n    │   └── domain.py           # module composed only domain logics\n    ├── poetry.lock             # poetry dependencies versions\n    ├── pyproject.toml          # poetry package definition file\n    ├── README.rst              # project documentation\n    └── tests                   # directory composed only with unit tests\n        └── ...\n',
    'author': 'Carlos Neto',
    'author_email': 'carlos.neto.dev@gmail.com',
    'maintainer': 'Carlos Neto',
    'maintainer_email': 'carlos.neto.dev@gmail.com',
    'url': 'https://github.com/augustoliks/cdn-test',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.6.1',
}


setup(**setup_kwargs)
